@@ -39,7 +39,9 @@ module Network.HTTP.Semantics.Client (
 
     -- * Aux
     Aux,
+    defaultAux,
     auxPossibleClientStreams,
+    auxSendPing,
 
     -- * Types
     Scheme,
