@@ -1,6 +1,3 @@
-{-# LANGUAGE NamedFieldPuns #-}
-{-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
 module Network.HTTP.Semantics.FillBuf (
@@ -25,7 +22,7 @@ import Data.ByteString.Internal
 import Data.Int (Int64)
 import Data.Maybe
 import Foreign.Ptr (plusPtr)
-import Network.ByteOrder
+import Network.ByteOrder hiding (start)
 import Network.HTTP.Semantics.Client
 
 ----------------------------------------------------------------
@@ -35,7 +32,7 @@ import Network.HTTP.Semantics.Client
 -- In @http2@ this will be used to construct a single HTTP2 @DATA@ frame
 -- (see discussion of the maximum number of bytes, below).
 type DynaNext =
-       Buffer
+    Buffer
     -- ^ Write buffer
     -> Int
     -- ^ Maximum number of bytes we are allowed to write
@@ -76,15 +73,14 @@ data StreamingChunk
 -- | Action to run prior to terminating the stream
 type CleanupStream = IO ()
 
-data IsEndOfStream =
-    -- | The stream is not yet terminated
-    NotEndOfStream
-
-    -- | The stream is terminated
-    --
-    -- In addition to indicating that the stream is terminated, we can also
-    -- specify an optional `Cleanup` handler to be run.
-  | EndOfStream (Maybe CleanupStream)
+data IsEndOfStream
+    = -- | The stream is not yet terminated
+      NotEndOfStream
+    | -- | The stream is terminated
+      --
+      -- In addition to indicating that the stream is terminated, we can also
+      -- specify an optional `Cleanup` handler to be run.
+      EndOfStream (Maybe CleanupStream)
 
 ----------------------------------------------------------------
 
@@ -94,11 +90,11 @@ fillBuilderBodyGetNext bb buf room = do
     return $ nextForBuilder len signal
 
 fillFileBodyGetNext
-    :: PositionRead -> FileOffset -> ByteCount -> IO () -> DynaNext
-fillFileBodyGetNext pread start bytecount refresh buf room = do
+    :: PositionRead -> FileOffset -> ByteCount -> Sentinel -> DynaNext
+fillFileBodyGetNext pread start bytecount sentinel buf room = do
     len <- pread start (mini room bytecount) buf
     let len' = fromIntegral len
-    return $ nextForFile len' pread (start + len) (bytecount - len) refresh
+    nextForFile len' pread (start + len) (bytecount - len) sentinel
 
 fillStreamBodyGetNext :: IO (Maybe StreamingChunk) -> DynaNext
 fillStreamBodyGetNext takeQ = loop 0
@@ -116,10 +112,10 @@ fillBufBuilderOne :: Int -> B.BufferWriter -> DynaNext
 fillBufBuilderOne minReq writer buf0 room = do
     if room >= minReq
         then do
-          (len, signal) <- writer buf0 room
-          return $ nextForBuilder len signal
+            (len, signal) <- writer buf0 room
+            return $ nextForBuilder len signal
         else do
-          return $ Next 0 True (Just $ fillBufBuilderOne minReq writer)
+            return $ Next 0 True (Just $ fillBufBuilderOne minReq writer)
 
 fillBufBuilderTwo :: ByteString -> B.BufferWriter -> DynaNext
 fillBufBuilderTwo bs writer buf0 room
@@ -202,10 +198,10 @@ runStreamingBuilder builder next = \total buf room -> do
         let enoughRoom = maybe True (room >=) mMinReq
         if enoughRoom
             then do
-              writeResult <- writer buf room
-              ranWriter writeResult total buf room
+                writeResult <- writer buf room
+                ranWriter writeResult total buf room
             else do
-              return $ Next total True (Just $ goMore mMinReq writer 0)
+                return $ Next total True (Just $ goMore mMinReq writer 0)
 
     goChunk :: ByteString -> B.BufferWriter -> NextWithTotal
     goChunk bs writer = \total buf room ->
@@ -221,19 +217,25 @@ runStreamingBuilder builder next = \total buf room -> do
 
 ----------------------------------------------------------------
 
-fillBufFile :: PositionRead -> FileOffset -> ByteCount -> IO () -> DynaNext
-fillBufFile pread start bytes refresh buf room = do
+fillBufFile :: PositionRead -> FileOffset -> ByteCount -> Sentinel -> DynaNext
+fillBufFile pread start bytes sentinel buf room = do
     len <- pread start (mini room bytes) buf
-    refresh
+    case sentinel of
+        Refresher refresh -> refresh
+        _ -> return ()
     let len' = fromIntegral len
-    return $ nextForFile len' pread (start + len) (bytes - len) refresh
+    nextForFile len' pread (start + len) (bytes - len) sentinel
 
 nextForFile
-    :: BytesFilled -> PositionRead -> FileOffset -> ByteCount -> IO () -> Next
-nextForFile 0 _ _ _ _ = Next 0 True Nothing -- let's flush
-nextForFile len _ _ 0 _ = Next len False Nothing
+    :: BytesFilled -> PositionRead -> FileOffset -> ByteCount -> Sentinel -> IO Next
+nextForFile 0 _ _ _ _ = return $ Next 0 True Nothing -- let's flush
+nextForFile len _ _ 0 sentinel = do
+    case sentinel of
+        Closer close -> close
+        _ -> return ()
+    return $ Next len False Nothing
 nextForFile len pread start bytes refresh =
-    Next len False $ Just $ fillBufFile pread start bytes refresh
+    return $ Next len False $ Just $ fillBufFile pread start bytes refresh
 
 {-# INLINE mini #-}
 mini :: Int -> Int64 -> Int64
